# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union

import httpx

from ...types import (
    lookup_table_list_params,
    lookup_table_create_params,
    lookup_table_update_params,
    lookup_table_retrieve_params,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from .lookup_table_revisions import (
    LookupTableRevisionsResource,
    AsyncLookupTableRevisionsResource,
    LookupTableRevisionsResourceWithRawResponse,
    AsyncLookupTableRevisionsResourceWithRawResponse,
    LookupTableRevisionsResourceWithStreamingResponse,
    AsyncLookupTableRevisionsResourceWithStreamingResponse,
)
from ...types.lookup_table_response import LookupTableResponse
from .lookup_table_revision_data.lookup_table_revision_data import (
    LookupTableRevisionDataResource,
    AsyncLookupTableRevisionDataResource,
    LookupTableRevisionDataResourceWithRawResponse,
    AsyncLookupTableRevisionDataResourceWithRawResponse,
    LookupTableRevisionDataResourceWithStreamingResponse,
    AsyncLookupTableRevisionDataResourceWithStreamingResponse,
)

__all__ = ["LookupTablesResource", "AsyncLookupTablesResource"]


class LookupTablesResource(SyncAPIResource):
    """Endpoints for creating/updating/deleting Lookup Tables.

    Lookup Tables enable you to manage dynamic data mappings that your calculations reference. Use them for currency conversion, pricing tiers, discount rates, and similar scenarios where you require values to change operationally but for calculation logic to remain constant.

    **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
    """

    @cached_property
    def lookup_table_revisions(self) -> LookupTableRevisionsResource:
        """Endpoints for creating/updating/deleting Lookup Table Revisions.

        Lookup Tables utilize a "Table and Revision" model, which lets you update data cleanly, and extend the schema without modifying existing calculations:
        * Create Revisions for a Lookup Table, which you can use to define data schema and lookup keys.
        * Populate draft Revisions with data values. You can create and edit multiple draft Revisions, but only one can be published at any given time.
        * Publish a Revision to activate it. When you use Lookup functions in your calculations that reference the Lookup Table, the data values defined for the published Revision are used.
        * When you want different, updated data values to be used, publish the draft Revision containing the required new values. The currently published Revision is archived automatically.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return LookupTableRevisionsResource(self._client)

    @cached_property
    def lookup_table_revision_data(self) -> LookupTableRevisionDataResource:
        """
        Endpoints for creating/updating/deleting Data for specific Lookup Table Revisions.

        When you've added fields to create a data schema for a Lookup Table Revision, you can use upsert operations to create or update the data values for those fields:
        * Use [Upsert LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionData) to upsert some or all of a Revision's field data values.
        * Use [Upsert LookupTableRevisionData Entry](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionDataEntry) to upsert an individual Revision field's data value.

        **NOTES:**
        * You can only create or update field data values for DRAFT Revisions.
        * You cannot change the field data values for PUBLISHED Revisions.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision Data endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return LookupTableRevisionDataResource(self._client)

    @cached_property
    def with_raw_response(self) -> LookupTablesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LookupTablesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LookupTablesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return LookupTablesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Create a new Lookup Table.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/lookuptables",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_create_params.LookupTableCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Retrieve a Lookup Table by UUID.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for the Lookup
              Tables returned, set `additional=activeRevision` in the query.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"additional": additional}, lookup_table_retrieve_params.LookupTableRetrieveParams
                ),
            ),
            cast_to=LookupTableResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Update the Lookup Table with the given UUID.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/lookuptables/{id}",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_update_params.LookupTableUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[LookupTableResponse]:
        """
        Retrieve a list Lookup Tables created for the Organization:

        - Returned list can be filtered by Lookup Table `code` query parameter.
        - If you want to include any non-default fields for the returned Lookup Tables,
          use the additional query parameter to specify which you want to include in the
          response.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for each of the
              Lookup Tables returned, set `additional=activeRevision` in the query.

          codes: List of Lookup Table codes to retrieve.

          next_token: Token to supply for multi page retrievals.

          page_size: Number of Lookup Tables to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables",
            page=SyncCursor[LookupTableResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional": additional,
                        "codes": codes,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_list_params.LookupTableListParams,
                ),
            ),
            model=LookupTableResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Delete the Lookup Table with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )


class AsyncLookupTablesResource(AsyncAPIResource):
    """Endpoints for creating/updating/deleting Lookup Tables.

    Lookup Tables enable you to manage dynamic data mappings that your calculations reference. Use them for currency conversion, pricing tiers, discount rates, and similar scenarios where you require values to change operationally but for calculation logic to remain constant.

    **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
    """

    @cached_property
    def lookup_table_revisions(self) -> AsyncLookupTableRevisionsResource:
        """Endpoints for creating/updating/deleting Lookup Table Revisions.

        Lookup Tables utilize a "Table and Revision" model, which lets you update data cleanly, and extend the schema without modifying existing calculations:
        * Create Revisions for a Lookup Table, which you can use to define data schema and lookup keys.
        * Populate draft Revisions with data values. You can create and edit multiple draft Revisions, but only one can be published at any given time.
        * Publish a Revision to activate it. When you use Lookup functions in your calculations that reference the Lookup Table, the data values defined for the published Revision are used.
        * When you want different, updated data values to be used, publish the draft Revision containing the required new values. The currently published Revision is archived automatically.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return AsyncLookupTableRevisionsResource(self._client)

    @cached_property
    def lookup_table_revision_data(self) -> AsyncLookupTableRevisionDataResource:
        """
        Endpoints for creating/updating/deleting Data for specific Lookup Table Revisions.

        When you've added fields to create a data schema for a Lookup Table Revision, you can use upsert operations to create or update the data values for those fields:
        * Use [Upsert LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionData) to upsert some or all of a Revision's field data values.
        * Use [Upsert LookupTableRevisionData Entry](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionDataEntry) to upsert an individual Revision field's data value.

        **NOTES:**
        * You can only create or update field data values for DRAFT Revisions.
        * You cannot change the field data values for PUBLISHED Revisions.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision Data endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return AsyncLookupTableRevisionDataResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLookupTablesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLookupTablesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLookupTablesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncLookupTablesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Create a new Lookup Table.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/lookuptables",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_create_params.LookupTableCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Retrieve a Lookup Table by UUID.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for the Lookup
              Tables returned, set `additional=activeRevision` in the query.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"additional": additional}, lookup_table_retrieve_params.LookupTableRetrieveParams
                ),
            ),
            cast_to=LookupTableResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Update the Lookup Table with the given UUID.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/lookuptables/{id}",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_update_params.LookupTableUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LookupTableResponse, AsyncCursor[LookupTableResponse]]:
        """
        Retrieve a list Lookup Tables created for the Organization:

        - Returned list can be filtered by Lookup Table `code` query parameter.
        - If you want to include any non-default fields for the returned Lookup Tables,
          use the additional query parameter to specify which you want to include in the
          response.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for each of the
              Lookup Tables returned, set `additional=activeRevision` in the query.

          codes: List of Lookup Table codes to retrieve.

          next_token: Token to supply for multi page retrievals.

          page_size: Number of Lookup Tables to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables",
            page=AsyncCursor[LookupTableResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional": additional,
                        "codes": codes,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_list_params.LookupTableListParams,
                ),
            ),
            model=LookupTableResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Delete the Lookup Table with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )


class LookupTablesResourceWithRawResponse:
    def __init__(self, lookup_tables: LookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = to_raw_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = to_raw_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = to_raw_response_wrapper(
            lookup_tables.update,
        )
        self.list = to_raw_response_wrapper(
            lookup_tables.list,
        )
        self.delete = to_raw_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> LookupTableRevisionsResourceWithRawResponse:
        """Endpoints for creating/updating/deleting Lookup Table Revisions.

        Lookup Tables utilize a "Table and Revision" model, which lets you update data cleanly, and extend the schema without modifying existing calculations:
        * Create Revisions for a Lookup Table, which you can use to define data schema and lookup keys.
        * Populate draft Revisions with data values. You can create and edit multiple draft Revisions, but only one can be published at any given time.
        * Publish a Revision to activate it. When you use Lookup functions in your calculations that reference the Lookup Table, the data values defined for the published Revision are used.
        * When you want different, updated data values to be used, publish the draft Revision containing the required new values. The currently published Revision is archived automatically.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return LookupTableRevisionsResourceWithRawResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> LookupTableRevisionDataResourceWithRawResponse:
        """
        Endpoints for creating/updating/deleting Data for specific Lookup Table Revisions.

        When you've added fields to create a data schema for a Lookup Table Revision, you can use upsert operations to create or update the data values for those fields:
        * Use [Upsert LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionData) to upsert some or all of a Revision's field data values.
        * Use [Upsert LookupTableRevisionData Entry](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionDataEntry) to upsert an individual Revision field's data value.

        **NOTES:**
        * You can only create or update field data values for DRAFT Revisions.
        * You cannot change the field data values for PUBLISHED Revisions.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision Data endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return LookupTableRevisionDataResourceWithRawResponse(self._lookup_tables.lookup_table_revision_data)


class AsyncLookupTablesResourceWithRawResponse:
    def __init__(self, lookup_tables: AsyncLookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = async_to_raw_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            lookup_tables.update,
        )
        self.list = async_to_raw_response_wrapper(
            lookup_tables.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> AsyncLookupTableRevisionsResourceWithRawResponse:
        """Endpoints for creating/updating/deleting Lookup Table Revisions.

        Lookup Tables utilize a "Table and Revision" model, which lets you update data cleanly, and extend the schema without modifying existing calculations:
        * Create Revisions for a Lookup Table, which you can use to define data schema and lookup keys.
        * Populate draft Revisions with data values. You can create and edit multiple draft Revisions, but only one can be published at any given time.
        * Publish a Revision to activate it. When you use Lookup functions in your calculations that reference the Lookup Table, the data values defined for the published Revision are used.
        * When you want different, updated data values to be used, publish the draft Revision containing the required new values. The currently published Revision is archived automatically.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return AsyncLookupTableRevisionsResourceWithRawResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> AsyncLookupTableRevisionDataResourceWithRawResponse:
        """
        Endpoints for creating/updating/deleting Data for specific Lookup Table Revisions.

        When you've added fields to create a data schema for a Lookup Table Revision, you can use upsert operations to create or update the data values for those fields:
        * Use [Upsert LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionData) to upsert some or all of a Revision's field data values.
        * Use [Upsert LookupTableRevisionData Entry](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionDataEntry) to upsert an individual Revision field's data value.

        **NOTES:**
        * You can only create or update field data values for DRAFT Revisions.
        * You cannot change the field data values for PUBLISHED Revisions.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision Data endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return AsyncLookupTableRevisionDataResourceWithRawResponse(self._lookup_tables.lookup_table_revision_data)


class LookupTablesResourceWithStreamingResponse:
    def __init__(self, lookup_tables: LookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = to_streamed_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            lookup_tables.update,
        )
        self.list = to_streamed_response_wrapper(
            lookup_tables.list,
        )
        self.delete = to_streamed_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> LookupTableRevisionsResourceWithStreamingResponse:
        """Endpoints for creating/updating/deleting Lookup Table Revisions.

        Lookup Tables utilize a "Table and Revision" model, which lets you update data cleanly, and extend the schema without modifying existing calculations:
        * Create Revisions for a Lookup Table, which you can use to define data schema and lookup keys.
        * Populate draft Revisions with data values. You can create and edit multiple draft Revisions, but only one can be published at any given time.
        * Publish a Revision to activate it. When you use Lookup functions in your calculations that reference the Lookup Table, the data values defined for the published Revision are used.
        * When you want different, updated data values to be used, publish the draft Revision containing the required new values. The currently published Revision is archived automatically.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return LookupTableRevisionsResourceWithStreamingResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> LookupTableRevisionDataResourceWithStreamingResponse:
        """
        Endpoints for creating/updating/deleting Data for specific Lookup Table Revisions.

        When you've added fields to create a data schema for a Lookup Table Revision, you can use upsert operations to create or update the data values for those fields:
        * Use [Upsert LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionData) to upsert some or all of a Revision's field data values.
        * Use [Upsert LookupTableRevisionData Entry](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionDataEntry) to upsert an individual Revision field's data value.

        **NOTES:**
        * You can only create or update field data values for DRAFT Revisions.
        * You cannot change the field data values for PUBLISHED Revisions.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision Data endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return LookupTableRevisionDataResourceWithStreamingResponse(self._lookup_tables.lookup_table_revision_data)


class AsyncLookupTablesResourceWithStreamingResponse:
    def __init__(self, lookup_tables: AsyncLookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = async_to_streamed_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            lookup_tables.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lookup_tables.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> AsyncLookupTableRevisionsResourceWithStreamingResponse:
        """Endpoints for creating/updating/deleting Lookup Table Revisions.

        Lookup Tables utilize a "Table and Revision" model, which lets you update data cleanly, and extend the schema without modifying existing calculations:
        * Create Revisions for a Lookup Table, which you can use to define data schema and lookup keys.
        * Populate draft Revisions with data values. You can create and edit multiple draft Revisions, but only one can be published at any given time.
        * Publish a Revision to activate it. When you use Lookup functions in your calculations that reference the Lookup Table, the data values defined for the published Revision are used.
        * When you want different, updated data values to be used, publish the draft Revision containing the required new values. The currently published Revision is archived automatically.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return AsyncLookupTableRevisionsResourceWithStreamingResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> AsyncLookupTableRevisionDataResourceWithStreamingResponse:
        """
        Endpoints for creating/updating/deleting Data for specific Lookup Table Revisions.

        When you've added fields to create a data schema for a Lookup Table Revision, you can use upsert operations to create or update the data values for those fields:
        * Use [Upsert LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionData) to upsert some or all of a Revision's field data values.
        * Use [Upsert LookupTableRevisionData Entry](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/PutLookupTableRevisionDataEntry) to upsert an individual Revision field's data value.

        **NOTES:**
        * You can only create or update field data values for DRAFT Revisions.
        * You cannot change the field data values for PUBLISHED Revisions.

        **Beta Version!** The Lookup Table feature is currently available in Beta release version. See [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages) for Beta release definition. Lookup Table Revision Data endpoints will only be available if Lookup Tables have been enabled for your Organization. For more details see [Lookup Tables (Beta)](https://www.m3ter.com/docs/guides/lookup-tables) in our main User documentation.
        """
        return AsyncLookupTableRevisionDataResourceWithStreamingResponse(self._lookup_tables.lookup_table_revision_data)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .statement_jobs import (
    StatementJobsResource,
    AsyncStatementJobsResource,
    StatementJobsResourceWithRawResponse,
    AsyncStatementJobsResourceWithRawResponse,
    StatementJobsResourceWithStreamingResponse,
    AsyncStatementJobsResourceWithStreamingResponse,
)
from .statement_definitions import (
    StatementDefinitionsResource,
    AsyncStatementDefinitionsResource,
    StatementDefinitionsResourceWithRawResponse,
    AsyncStatementDefinitionsResourceWithRawResponse,
    StatementDefinitionsResourceWithStreamingResponse,
    AsyncStatementDefinitionsResourceWithStreamingResponse,
)
from ...types.object_url_response import ObjectURLResponse

__all__ = ["StatementsResource", "AsyncStatementsResource"]


class StatementsResource(SyncAPIResource):
    """
    Endpoints for billing operations such as creating, updating, listing,downloading, and deleting Bills.

    Bills are generated for an Account, and are calculated in accordance with the usage-based pricing Plans applied for the Products the Account consumes. These endpoints enable interaction with the billing system, allowing you to obtain billing details and insights into the consumption patterns and charges of your end-customer Accounts.
    """

    @cached_property
    def statement_jobs(self) -> StatementJobsResource:
        """Endpoints for creating, retrieving, listing, and cancelling statement jobs.

        StatementJobs are tasks to asynchronously calculate and generate a bill statement.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.
        """
        return StatementJobsResource(self._client)

    @cached_property
    def statement_definitions(self) -> StatementDefinitionsResource:
        """
        Endpoints for listing, creating, updating, retrieving, or deleting Statement Definitions.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.

        Statement Definitions specify the way billed usage will be aggregated and compiled in the Statement. For example, if you are billing customers monthly, you might want to breakdown the usage responsible for the monthly charge on a Bill into weekly portions in Bill statements.
        """
        return StatementDefinitionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> StatementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return StatementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return StatementsResourceWithStreamingResponse(self)

    def create_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Generate a specific Bill Statement for the provided Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you then use
        with a `GET` call to obtain the Bill statement in CSV format.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    def get_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a specific Bill Statement for the given Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response includes a pre-signed `downloadUrl`, which must be used with a
        separate `GET` call to download the actual Bill Statement. This ensures secure
        access to the requested information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    def get_json(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a Bill Statement in JSON format for a given Bill ID.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you use with
        a `GET` call to obtain the Bill Statement.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/{id}/statement/json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )


class AsyncStatementsResource(AsyncAPIResource):
    """
    Endpoints for billing operations such as creating, updating, listing,downloading, and deleting Bills.

    Bills are generated for an Account, and are calculated in accordance with the usage-based pricing Plans applied for the Products the Account consumes. These endpoints enable interaction with the billing system, allowing you to obtain billing details and insights into the consumption patterns and charges of your end-customer Accounts.
    """

    @cached_property
    def statement_jobs(self) -> AsyncStatementJobsResource:
        """Endpoints for creating, retrieving, listing, and cancelling statement jobs.

        StatementJobs are tasks to asynchronously calculate and generate a bill statement.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.
        """
        return AsyncStatementJobsResource(self._client)

    @cached_property
    def statement_definitions(self) -> AsyncStatementDefinitionsResource:
        """
        Endpoints for listing, creating, updating, retrieving, or deleting Statement Definitions.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.

        Statement Definitions specify the way billed usage will be aggregated and compiled in the Statement. For example, if you are billing customers monthly, you might want to breakdown the usage responsible for the monthly charge on a Bill into weekly portions in Bill statements.
        """
        return AsyncStatementDefinitionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncStatementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncStatementsResourceWithStreamingResponse(self)

    async def create_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Generate a specific Bill Statement for the provided Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you then use
        with a `GET` call to obtain the Bill statement in CSV format.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    async def get_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a specific Bill Statement for the given Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response includes a pre-signed `downloadUrl`, which must be used with a
        separate `GET` call to download the actual Bill Statement. This ensures secure
        access to the requested information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    async def get_json(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a Bill Statement in JSON format for a given Bill ID.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you use with
        a `GET` call to obtain the Bill Statement.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/{id}/statement/json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )


class StatementsResourceWithRawResponse:
    def __init__(self, statements: StatementsResource) -> None:
        self._statements = statements

        self.create_csv = to_raw_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = to_raw_response_wrapper(
            statements.get_csv,
        )
        self.get_json = to_raw_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> StatementJobsResourceWithRawResponse:
        """Endpoints for creating, retrieving, listing, and cancelling statement jobs.

        StatementJobs are tasks to asynchronously calculate and generate a bill statement.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.
        """
        return StatementJobsResourceWithRawResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> StatementDefinitionsResourceWithRawResponse:
        """
        Endpoints for listing, creating, updating, retrieving, or deleting Statement Definitions.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.

        Statement Definitions specify the way billed usage will be aggregated and compiled in the Statement. For example, if you are billing customers monthly, you might want to breakdown the usage responsible for the monthly charge on a Bill into weekly portions in Bill statements.
        """
        return StatementDefinitionsResourceWithRawResponse(self._statements.statement_definitions)


class AsyncStatementsResourceWithRawResponse:
    def __init__(self, statements: AsyncStatementsResource) -> None:
        self._statements = statements

        self.create_csv = async_to_raw_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = async_to_raw_response_wrapper(
            statements.get_csv,
        )
        self.get_json = async_to_raw_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> AsyncStatementJobsResourceWithRawResponse:
        """Endpoints for creating, retrieving, listing, and cancelling statement jobs.

        StatementJobs are tasks to asynchronously calculate and generate a bill statement.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.
        """
        return AsyncStatementJobsResourceWithRawResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> AsyncStatementDefinitionsResourceWithRawResponse:
        """
        Endpoints for listing, creating, updating, retrieving, or deleting Statement Definitions.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.

        Statement Definitions specify the way billed usage will be aggregated and compiled in the Statement. For example, if you are billing customers monthly, you might want to breakdown the usage responsible for the monthly charge on a Bill into weekly portions in Bill statements.
        """
        return AsyncStatementDefinitionsResourceWithRawResponse(self._statements.statement_definitions)


class StatementsResourceWithStreamingResponse:
    def __init__(self, statements: StatementsResource) -> None:
        self._statements = statements

        self.create_csv = to_streamed_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = to_streamed_response_wrapper(
            statements.get_csv,
        )
        self.get_json = to_streamed_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> StatementJobsResourceWithStreamingResponse:
        """Endpoints for creating, retrieving, listing, and cancelling statement jobs.

        StatementJobs are tasks to asynchronously calculate and generate a bill statement.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.
        """
        return StatementJobsResourceWithStreamingResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> StatementDefinitionsResourceWithStreamingResponse:
        """
        Endpoints for listing, creating, updating, retrieving, or deleting Statement Definitions.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.

        Statement Definitions specify the way billed usage will be aggregated and compiled in the Statement. For example, if you are billing customers monthly, you might want to breakdown the usage responsible for the monthly charge on a Bill into weekly portions in Bill statements.
        """
        return StatementDefinitionsResourceWithStreamingResponse(self._statements.statement_definitions)


class AsyncStatementsResourceWithStreamingResponse:
    def __init__(self, statements: AsyncStatementsResource) -> None:
        self._statements = statements

        self.create_csv = async_to_streamed_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = async_to_streamed_response_wrapper(
            statements.get_csv,
        )
        self.get_json = async_to_streamed_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> AsyncStatementJobsResourceWithStreamingResponse:
        """Endpoints for creating, retrieving, listing, and cancelling statement jobs.

        StatementJobs are tasks to asynchronously calculate and generate a bill statement.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.
        """
        return AsyncStatementJobsResourceWithStreamingResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> AsyncStatementDefinitionsResourceWithStreamingResponse:
        """
        Endpoints for listing, creating, updating, retrieving, or deleting Statement Definitions.

        Bill statements are informative backing sheets to invoices. They provide a breakdown of the usage charges that appear on the bill, helping your end customers better understand those charges, and gain a clearer picture of their usage over the billing period.

        Statement Definitions specify the way billed usage will be aggregated and compiled in the Statement. For example, if you are billing customers monthly, you might want to breakdown the usage responsible for the monthly charge on a Bill into weekly portions in Bill statements.
        """
        return AsyncStatementDefinitionsResourceWithStreamingResponse(self._statements.statement_definitions)
